
#include "../src/main.h"
#include "../src/game.h"

#define NUM_DXBOULDER_LEVELS	100
#define DX_MAP_HEADER_SIZE	398
#define DX_LEVEL_HEADER_SIZE	358

char *progname;

struct LevelInfo level;
short Ur[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
int MampferMax;
int lev_fieldx, lev_fieldy;

#define EL_UNKNOWN_EL			-999

#if 1
#undef  EL_DX_UNKNOWN_15
#define EL_DX_UNKNOWN_15	EL_UNKNOWN_EL
#undef  EL_DX_UNKNOWN_42
#define EL_DX_UNKNOWN_42	EL_UNKNOWN_EL
#endif

#define BYTE_ORDER_BIG_ENDIAN		0
#define BYTE_ORDER_LITTLE_ENDIAN	1

void putFile16BitInteger(FILE *file, short value, int byte_order)
{
  if (byte_order == BYTE_ORDER_BIG_ENDIAN)
  {
    fputc((value >>  8) & 0xff, file);
    fputc((value >>  0) & 0xff, file);
  }
  else		 /* BYTE_ORDER_LITTLE_ENDIAN */
  {
    fputc((value >>  0) & 0xff, file);
    fputc((value >>  8) & 0xff, file);
  }
}

void putFileChunk(FILE *file, char *chunk_name, int chunk_length,
		  int byte_order)
{
  fputs(chunk_name, file);

  if (byte_order == BYTE_ORDER_BIG_ENDIAN)
  {
    fputc((chunk_length >> 24) & 0xff, file);
    fputc((chunk_length >> 16) & 0xff, file);
    fputc((chunk_length >>  8) & 0xff, file);
    fputc((chunk_length >>  0) & 0xff, file);
  }
  else		 /* BYTE_ORDER_LITTLE_ENDIAN */
  {
    fputc((chunk_length >>  0) & 0xff, file);
    fputc((chunk_length >>  8) & 0xff, file);
    fputc((chunk_length >> 16) & 0xff, file);
    fputc((chunk_length >> 24) & 0xff, file);
  }
}


#define LEVEL_HEADER_SIZE	80	/* size of level file header */
#define LEVEL_HEADER_UNUSED	15	/* unused level header bytes */

/* file identifier strings */
#define LEVEL_COOKIE_12		"ROCKSNDIAMONDS_LEVEL_FILE_VERSION_1.2"
#define LEVEL_COOKIE		"ROCKSNDIAMONDS_LEVEL_FILE_VERSION_1.4"

/* file permissions for newly written files */
#define MODE_R_ALL		(S_IRUSR | S_IRGRP | S_IROTH)
#define MODE_W_ALL		(S_IWUSR | S_IWGRP | S_IWOTH)
#define MODE_X_ALL		(S_IXUSR | S_IXGRP | S_IXOTH)
#define LEVEL_PERMS		(MODE_R_ALL | MODE_W_ALL)

#if 0
void OLD_SaveLevel(char *filename)
{
  int i, x, y;
  FILE *file;

  if (!(file = fopen(filename, "w")))
  {
    fprintf(stderr, "%s: cannot save level file '%s'\n", progname, filename);
    exit(1);
  }

  fputs(LEVEL_COOKIE, file);		/* file identifier */
  fputc('\n', file);

  putFileChunk(file, "HEAD", LEVEL_HEADER_SIZE, BYTE_ORDER_BIG_ENDIAN);

  fputc(level.fieldx, file);
  fputc(level.fieldy, file);
  fputc(level.time / 256, file);
  fputc(level.time % 256, file);
  fputc(level.gems_needed / 256, file);
  fputc(level.gems_needed % 256, file);

  for(i=0; i<MAX_LEVEL_NAME_LEN; i++)
    fputc(level.name[i], file);
  for(i=0; i<LEVEL_SCORE_ELEMENTS; i++)
    fputc(level.score[i], file);
  for(i=0; i<4; i++)
    for(y=0; y<3; y++)
      for(x=0; x<3; x++)
	fputc(level.yam_content[i][x][y], file);
  fputc(level.amoeba_speed, file);
  fputc(level.time_magic_wall, file);
  fputc(level.time_wheel, file);
  fputc(level.amoeba_content, file);
  fputc((level.double_speed ? 1 : 0), file);
  fputc((level.gravity ? 1 : 0), file);

  for(i=0; i<LEVEL_HEADER_UNUSED; i++)	/* set unused header bytes to zero */
    fputc(0, file);

  putFileChunk(file, "AUTH", MAX_LEVEL_AUTHOR_LEN, BYTE_ORDER_BIG_ENDIAN);

  for(i=0; i<MAX_LEVEL_AUTHOR_LEN; i++)
    fputc(level.author[i], file);

  putFileChunk(file, "CONT", 4 + 8 * 3 * 3, BYTE_ORDER_BIG_ENDIAN);

  fputc(EL_MAMPFER, file);
  fputc(MampferMax, file);
  fputc(0, file);
  fputc(0, file);

  for(i=0; i<8; i++)
    for(y=0; y<3; y++)
      for(x=0; x<3; x++)
	fputc(level.yam_content[i][x][y], file);

  putFileChunk(file, "BODY", lev_fieldx * lev_fieldy, BYTE_ORDER_BIG_ENDIAN);

  for(y=0; y<lev_fieldy; y++) 
    for(x=0; x<lev_fieldx; x++) 
      fputc(Ur[x][y], file);

  fclose(file);

  chmod(filename, LEVEL_PERMS);
}
#endif

void SaveLevel(char *filename)
{
  int i, x, y;
  boolean encoding_16bit = FALSE;	/* default: maximal 256 elements */
  char *oldest_possible_cookie;
  FILE *file;

  if (!(file = fopen(filename, "w")))
  {
    fprintf(stderr, "%s: cannot save level file '%s'\n", progname, filename);
    exit(1);
  }

  /* check yam content for 16-bit elements */
  for(i=0; i<MAX_ELEMENT_CONTENTS; i++)
    for(y=0; y<3; y++)
      for(x=0; x<3; x++)
	if (level.yam_content[i][x][y] > 255)
	  encoding_16bit = TRUE;

  /* check level field for 16-bit elements */
  for(y=0; y<lev_fieldy; y++) 
    for(x=0; x<lev_fieldx; x++) 
      if (Ur[x][y] > 255)
	encoding_16bit = TRUE;

  oldest_possible_cookie = (encoding_16bit ? LEVEL_COOKIE : LEVEL_COOKIE_12);

  fputs(oldest_possible_cookie, file);		/* file identifier */
  fputc('\n', file);

  putFileChunk(file, "HEAD", LEVEL_HEADER_SIZE, BYTE_ORDER_BIG_ENDIAN);

  fputc(level.fieldx, file);
  fputc(level.fieldy, file);

  putFile16BitInteger(file, level.time, BYTE_ORDER_BIG_ENDIAN);
  putFile16BitInteger(file, level.gems_needed, BYTE_ORDER_BIG_ENDIAN);

  for(i=0; i<MAX_LEVEL_NAME_LEN; i++)
    fputc(level.name[i], file);
  for(i=0; i<LEVEL_SCORE_ELEMENTS; i++)
    fputc(level.score[i], file);
  for(i=0; i<STD_ELEMENT_CONTENTS; i++)
    for(y=0; y<3; y++)
      for(x=0; x<3; x++)
	fputc(encoding_16bit ? EL_LEERRAUM : level.yam_content[i][x][y], file);
  fputc(level.amoeba_speed, file);
  fputc(level.time_magic_wall, file);
  fputc(level.time_wheel, file);
  fputc(level.amoeba_content, file);
  fputc((level.double_speed ? 1 : 0), file);
  fputc((level.gravity ? 1 : 0), file);

  fputc((encoding_16bit ? 1 : 0), file);

  for(i=0; i<LEVEL_HEADER_UNUSED; i++)	/* set unused header bytes to zero */
    fputc(0, file);

  putFileChunk(file, "AUTH", MAX_LEVEL_AUTHOR_LEN, BYTE_ORDER_BIG_ENDIAN);

  for(i=0; i<MAX_LEVEL_AUTHOR_LEN; i++)
    fputc(level.author[i], file);

  putFileChunk(file, "CONT", 4 + MAX_ELEMENT_CONTENTS * 3 * 3,
	       BYTE_ORDER_BIG_ENDIAN);

  fputc(EL_MAMPFER, file);
  fputc(level.num_yam_contents, file);
  fputc(0, file);
  fputc(0, file);

  for(i=0; i<MAX_ELEMENT_CONTENTS; i++)
    for(y=0; y<3; y++)
      for(x=0; x<3; x++)
	if (encoding_16bit)
	  putFile16BitInteger(file, level.yam_content[i][x][y],
			      BYTE_ORDER_BIG_ENDIAN);
	else
	  fputc(level.yam_content[i][x][y], file);

  putFileChunk(file, "BODY", lev_fieldx * lev_fieldy, BYTE_ORDER_BIG_ENDIAN);

  for(y=0; y<lev_fieldy; y++) 
    for(x=0; x<lev_fieldx; x++) 
      if (encoding_16bit)
	putFile16BitInteger(file, Ur[x][y], BYTE_ORDER_BIG_ENDIAN);
      else
	fputc(Ur[x][y], file);

  fclose(file);

  chmod(filename, LEVEL_PERMS);
}


static void skip_bytes(FILE *file, int skip_bytes)
{
  int i;

  for (i=0; i<skip_bytes; i++)
    fgetc(file);
}

int map_dx_element(int element)
{
  switch (element)
  {
    case 0:	return EL_LEERRAUM;
    case 1:	return EL_ERDREICH;
    case 2:	return EL_SPIELFIGUR;
    case 3:	return EL_FELSBROCKEN;
    case 4:	return EL_EDELSTEIN;		/* "diamond" */
    case 5:	return EL_KAEFER_LEFT;
    case 6:	return EL_BETON;
    case 7:	return EL_AUSGANG_ZU;
    case 8:	return EL_AUSGANG_AUF;
    case 9:	return EL_BOMBE;
    case 10:	return EL_DYNAMITE_ACTIVE;	/* explosion */
    case 11:	return EL_AMOEBE_NORM;		/* amoeba */
    case 12:	return EL_KOKOSNUSS;
    case 13:	return EL_DIAMANT;		/* "emerald" */
    case 14:	return EL_MAMPFER;
    case 15:	return EL_UNKNOWN_EL;		/* !!! grover !!! */
    case 16:	return EL_MAGIC_WALL_OFF;
    case 17:	return EL_BD_WALL;		/* crumble wall 1 */
    case 18:	return EL_UNKNOWN_EL;		/* magic wall with rock */
    case 19:	return EL_SP_TERMINAL;
    case 20:	return EL_MAUERWERK;		/* pattern 1 (wall) */
    case 21:	return EL_MAUERWERK;		/* pattern 2 (wall) */
    case 22:	return EL_MAUERWERK;		/* pattern 3 (wall) */
    case 23:	return EL_MAUERWERK;		/* pattern 4 (wall) */
    case 24:	return EL_MAUERWERK;		/* pattern 5 (wall) */
    case 25:	return EL_MAUERWERK;		/* pattern 6 (wall) */
    case 26:	return EL_MAUERWERK;		/* pattern 7 (wall) */
    case 27:	return EL_MAUERWERK;		/* pattern 8 (wall) */
    case 28:	return EL_BETON;		/* pattern 9 (steel wall) */
    case 29:	return EL_BETON;		/* pattern 10 (steel wall) */
    case 30:	return EL_AUSGANG_ZU;		/* !!! C64 BD Hidden Exit !!!*/
    case 31:	return EL_FELSBODEN;		/* crumble wall 2 */
    case 32:	return EL_FELSBODEN;		/* crumble wall 3 */
    case 33:	return EL_FELSBODEN;		/* crumble wall 4 */
    case 34:	return EL_FELSBODEN;		/* crumble wall 5 */
    case 35:	return EL_SP_PORT1_LEFT;
    case 36:	return EL_SP_PORT1_RIGHT;
    case 37:	return EL_SP_PORT1_UP;
    case 38:	return EL_SP_PORT1_DOWN;
    case 39:	return EL_SP_PORT_Y;
    case 40:	return EL_SP_PORT_X;
    case 41:	return EL_SP_PORT_XY;
    case 42:	return EL_UNKNOWN_EL;		/* !!! Slime !!! */
    case 43:	return EL_SP_PORT2_LEFT;	/* Tube Left toggle Gravity */
    case 44:	return EL_SP_PORT2_RIGHT	/* Tube Rght toggle Gravity */;
    case 45:	return EL_SP_PORT2_UP;		/* Tube Up toggle Gravity */
    case 46:	return EL_SP_PORT2_DOWN;	/* Tube Down toggle Gravity */
    case 47:	return EL_UNKNOWN_EL;		/* !!! Tube Green !!! */
    case 48:	return EL_UNKNOWN_EL;		/* !!! Tube Blue !!! */
    case 49:	return EL_UNKNOWN_EL;		/* !!! Tube Yellow !!! */
    case 50:	return EL_UNKNOWN_EL;		/* !!! Tube Red !!! */
    case 51:	return EL_UNKNOWN_EL;		/* !!! Tube Gray !!! */
    case 52:	return EL_EM_GATE_3;
    case 53:	return EL_EM_GATE_4;
    case 54:	return EL_EM_GATE_2;
    case 55:	return EL_EM_GATE_1;
    case 56:	return EL_EM_GATE_3X;
    case 57:	return EL_EM_KEY_3_FILE;
    case 58:	return EL_EM_KEY_4_FILE;
    case 59:	return EL_EM_KEY_2_FILE;
    case 60:	return EL_EM_KEY_1_FILE;
    case 61:	return EL_ROBOT;
    case 62:	return EL_SALZSAEURE;
    case 63:	return EL_BADEWANNE1;
    case 64:	return EL_BADEWANNE2;
    case 65:	return EL_BADEWANNE3;
    case 66:	return EL_BADEWANNE5;
    case 67:	return EL_BADEWANNE4;
    case 68:	return EL_UNSICHTBAR;
    case 69:	return EL_MORAST_VOLL;
    case 70:	return EL_FLIEGER_LEFT;
    case 71:	return EL_DYNAMITE_INACTIVE;
    case 72:	return EL_ABLENK_AUS;
    case 73:	return EL_DYNAMITE_ACTIVE;
    case 74:	return EL_AMOEBE_NASS;		/* amoeba drop */
    case 75:	return EL_AMOEBE_TOT;		/* amoeba stationary */
    case 76:	return EL_UNKNOWN_EL;		/* !!! Fixed Acid !!! */
    case 77:	return EL_MORAST_LEER;
    case 78:	return EL_UNKNOWN_EL;   	/* !!! RockBirth !!! */
    case 79:	return EL_EM_GATE_4X;
    case 80:	return EL_EM_GATE_2X;
    case 81:	return EL_EM_GATE_1X;
    case 82:	return EL_KAEFER_UP;
    case 83:	return EL_KAEFER_RIGHT;
    case 84:	return EL_KAEFER_DOWN;
    case 85:	return EL_FLIEGER_UP;
    case 86:	return EL_FLIEGER_RIGHT;
    case 87:	return EL_FLIEGER_DOWN;
    case 88:	return EL_MAMPFER;		/* left */
    case 89:	return EL_MAMPFER;		/* up */
    case 90:	return EL_MAMPFER;		/* right */
    case 91:	return EL_MAMPFER;		/* down */
    case 92:	return EL_SP_DISK_YELLOW;	/* utility disk */
    case 93:	return EL_SP_ELECTRON;		/* left */
    case 94:	return EL_SP_ELECTRON;		/* up */
    case 95:	return EL_SP_ELECTRON;		/* right */
    case 96:	return EL_SP_ELECTRON;		/* down */
    case 97:	return EL_TRAP_INACTIVE;   	/* !!! Trap Inactive !!! */
    case 98:	return EL_UNKNOWN_EL;   	/* !!! Trap Active !!! */
    case 99:	return EL_DX_SUPABOMB;   	/* !!! SupaBomb !!! */
    case 100:	return EL_CHAR('A');
    case 101:	return EL_CHAR('B');
    case 102:	return EL_CHAR('C');
    case 103:	return EL_CHAR('D');
    case 104:	return EL_CHAR('E');
    case 105:	return EL_CHAR('F');
    case 106:	return EL_CHAR('G');
    case 107:	return EL_CHAR('H');
    case 108:	return EL_CHAR('I');
    case 109:	return EL_CHAR('J');
    case 110:	return EL_CHAR('K');
    case 111:	return EL_CHAR('L');
    case 112:	return EL_CHAR('M');
    case 113:	return EL_CHAR('N');
    case 114:	return EL_CHAR('O');
    case 115:	return EL_CHAR('P');
    case 116:	return EL_CHAR('Q');
    case 117:	return EL_CHAR('R');
    case 118:	return EL_CHAR('S');
    case 119:	return EL_CHAR('T');
    case 120:	return EL_CHAR('U');
    case 121:	return EL_CHAR('V');
    case 122:	return EL_CHAR('W');
    case 123:	return EL_CHAR('X');
    case 124:	return EL_CHAR('Y');
    case 125:	return EL_CHAR('Z');
    case 126:	return EL_CHAR('0');
    case 127:	return EL_CHAR('1');
    case 128:	return EL_CHAR('2');
    case 129:	return EL_CHAR('3');
    case 130:	return EL_CHAR('4');
    case 131:	return EL_CHAR('5');
    case 132:	return EL_CHAR('6');
    case 133:	return EL_CHAR('7');
    case 134:	return EL_CHAR('8');
    case 135:	return EL_CHAR('9');
    case 136:	return EL_CHAR('.');
    case 137:	return EL_CHAR(',');
    case 138:	return EL_CHAR('?');
    case 139:	return EL_CHAR('!');
    case 140:	return EL_CHAR('/');
    case 141:	return EL_CHAR('<');
    case 142:	return EL_PFEIL_UP;
    case 143:	return EL_CHAR('>');
    case 144:	return EL_PFEIL_DOWN;
    case 145:	return EL_CHAR_COPY;
    case 146:	return EL_UNKNOWN_EL;   /* !!! Door Gray (Black) Closed !!! */
    case 147:	return EL_UNKNOWN_EL;   /* !!! Door Gray (Purple) Closed !!! */
    case 148:	return EL_UNKNOWN_EL;   /* !!! Door Gray (Brown) Closed !!! */
    case 149:	return EL_UNKNOWN_EL;   /* !!! Door Gray (White) Closed !!! */
    case 150:	return EL_SP_SNIKSNAK;		/* left */
    case 151:	return EL_SP_SNIKSNAK;		/* up */
    case 152:	return EL_SP_SNIKSNAK;		/* right */
    case 153:	return EL_SP_SNIKSNAK;		/* down */
    case 154:	return EL_MAUER_XY;
    case 155:	return EL_MAUER_X;
    case 156:	return EL_MAUER_Y;
    case 157:	return EL_UNKNOWN_EL;   	/* !!! Glue !!! */
    case 158:	return EL_TUBE_CROSS;   	/* tube cross */
    case 159:	return EL_TUBE_VERTICAL;   	/* tube vertical */
    case 160:	return EL_TUBE_HORIZONTAL;   	/* tube horizontal */
    case 161:	return EL_TUBE_VERT_LEFT;   	/* tube T left */
    case 162:	return EL_TUBE_HORIZ_UP;   	/* tube T up */
    case 163:	return EL_TUBE_VERT_RIGHT;   	/* tube T right */
    case 164:	return EL_TUBE_HORIZ_DOWN;   	/* tube T down */
    case 165:	return EL_TUBE_LEFT_UP;   	/* tube turn top left */
    case 166:	return EL_TUBE_RIGHT_UP;   	/* tube turn top right */
    case 167:	return EL_TUBE_LEFT_DOWN;   	/* tube turn bottom left */
    case 168:	return EL_TUBE_RIGHT_DOWN;   	/* tube turn bottom right */
    case 169:	return EL_UNKNOWN_EL;   	/* glue fellow */
    case 170:	return EL_PEARL;	   	/* pearl (DC) */
    case 171:	return EL_CRYSTAL;	   	/* crystal (DC) */
    case 172:	return EL_ERZ_EDEL;   		/* "diamond" wall */
    case 173:	return EL_ERZ_DIAM;   		/* "emerald" wall */
    case 174:	return EL_WALL_PEARL;   	/* pearl wall (DC) */
    case 175:	return EL_WALL_CRYSTAL;   	/* crystal wall (DC) */
    case 176:	return EL_DOOR_WHITE;   	/* white door (closed) (DC) */
    case 177:	return EL_DOOR_WHITE;   	/* white door (open) (DC) */
    case 178:	return EL_DOOR_WHITE_GRAY;   	/* gray/white door (DC) */
    case 179:	return EL_KEY_WHITE;	   	/* white key (DC) */
    case 180:	return EL_SHIELD_ACTIVE; 	/* force field (DC) */
    case 181:	return EL_EXTRA_TIME;   	/* extra time (10s) (DC) */
    case 182:	return EL_SWITCHGATE_OPEN;   	/* switch gate (open) (DC) */
    case 183:	return EL_SWITCHGATE_CLOSED;  	/* switch gate (closed) (DC) */
    case 184:	return EL_SWITCHGATE_SWITCH_1; 	/* switch (switch gate) (DC) */
    case 185:	return EL_TIMEGATE_CLOSED;	/* time gate (DC) */
    case 186:	return EL_TIMEGATE_SWITCH_OFF; 	/* time gate with wheel (DC) */
    case 187:	return EL_BELT3_LEFT;   	/* conveyor belt green left */
    case 188:	return EL_BELT3_MIDDLE;   	/* conv. belt green middle */
    case 189:	return EL_BELT3_RIGHT;   	/* conv. belt green right */
    case 190:	return EL_BELT3_SWITCH_MIDDLE; 	/* conv. belt green switch */
    case 191:	return EL_BELT1_LEFT;   	/* conveyor belt red left */
    case 192:	return EL_BELT1_MIDDLE;   	/* conv. belt red middle */
    case 193:	return EL_BELT1_RIGHT;   	/* conv. belt red right */
    case 194:	return EL_BELT1_SWITCH_MIDDLE; 	/* conv. belt red switch */
    case 195:	return EL_BD_ROCK;	   	/* C64 Boulder */
    case 196:	return EL_EDELSTEIN_BD;   	/* C64 Diamond */
    case 197:	return EL_UNKNOWN_EL;   	/* !!! Frozen life !!! */
    case 198:	return EL_UNKNOWN_EL;	 	/* !!! Poison Dirt !!! */
    case 199:	return EL_LANDMINE;   		/* mine (DC) */
    case 200:	return EL_ENVELOPE;   		/* envelope (DC) */
    case 201:	return EL_LIGHT_SWITCH_OFF;   	/* light switch (DC) */
    case 202:	return EL_UNKNOWN_EL;   	/* !!! 0 small !!! */
    case 203:	return EL_UNKNOWN_EL;   	/* !!! 1 small !!! */
    case 204:	return EL_UNKNOWN_EL;   	/* !!! 2 small !!! */
    case 205:	return EL_UNKNOWN_EL;   	/* !!! 3 small !!! */
    case 206:	return EL_UNKNOWN_EL;   	/* !!! 4 small !!! */
    case 207:	return EL_UNKNOWN_EL;   	/* !!! 5 small !!! */
    case 208:	return EL_UNKNOWN_EL;   	/* !!! 6 small !!! */
    case 209:	return EL_UNKNOWN_EL;   	/* !!! 7 small !!! */
    case 210:	return EL_UNKNOWN_EL;   	/* !!! 8 small !!! */
    case 211:	return EL_UNKNOWN_EL;   	/* !!! 9 small !!! */
    case 212:	return EL_UNKNOWN_EL;   	/* !!! Key Purple !!! */
    case 213:	return EL_UNKNOWN_EL;   	/* !!! Door Purple Closed !!!*/
    case 214:	return EL_UNKNOWN_EL;   	/* !!! Door Purple Open !!! */
    case 215:	return EL_UNKNOWN_EL;   	/* !!! Switch Well !!! */
    case 216:	return EL_UNKNOWN_EL;   	/* !!! Well !!! */
    case 217:	return EL_UNKNOWN_EL;   	/* !!! Candy-Trick !!! */
    case 218:	return EL_UNKNOWN_EL;   	/* !!! Door Brown Open !!! */
    case 219:	return EL_UNKNOWN_EL;   	/* !!! Door Brown Closed !!! */
    case 220:	return EL_UNKNOWN_EL;   	/* !!! Time Gate Open !!! */
    case 221:	return EL_UNKNOWN_EL;   	/* !!! SpringBounce !!! */
    case 222:	return EL_UNKNOWN_EL;   	/* !!! Door Black open !!! */
    case 223:	return EL_UNKNOWN_EL;   	/* !!! Door Black Closed !!! */
    case 224:	return EL_SONDE;	   	/* !!! Stitch !!! */
    case 225:	return EL_STEEL_SLANTED;   	/* Steel Slanted (DC) */
    case 226:	return EL_SAND_INVISIBLE;   	/* Dirt invisible (DC) */
    case 227:	return EL_INVISIBLE_STEEL;   	/* Steel invisible (DC) */
    case 228:	return EL_UNKNOWN_EL;   	/* !!! Hatch !!! */
    case 229:	return EL_FIREFLY_LEFT;
    case 230:	return EL_FIREFLY_UP;
    case 231:	return EL_FIREFLY_RIGHT;
    case 232:	return EL_FIREFLY_DOWN;
    case 233:	return EL_BUTTERFLY_LEFT;
    case 234:	return EL_BUTTERFLY_UP;
    case 235:	return EL_BUTTERFLY_RIGHT;
    case 236:	return EL_BUTTERFLY_DOWN;
    case 237:	return EL_SPIELER2;
    case 238:	return EL_PACMAN_LEFT;
    case 239:	return EL_PACMAN_UP;
    case 240:	return EL_PACMAN_RIGHT;
    case 241:	return EL_PACMAN_DOWN;
    case 242:	return EL_LIFE_ASYNC;   	/* !!! Biomaze !!! */
    case 243:	return EL_BALLOON;
    case 244:	return EL_BALLOON_SEND_ANY;	/* Wind cross */
    case 245:	return EL_BALLOON_SEND_LEFT;
    case 246:	return EL_BALLOON_SEND_UP;
    case 247:	return EL_BALLOON_SEND_RIGHT;
    case 248:	return EL_BALLOON_SEND_DOWN;
    case 249:	return EL_UNKNOWN_EL;   	/* !!! Wind Still !!! */
    case 250:	return EL_SPRING; 	  	/* !!! Spring !!! */
    case 251:	return EL_UNKNOWN_EL;   	/* !!! Glasses !!! */
    case 252:	return EL_UNKNOWN_EL;   	/* !!! Key Gray !!! */
    case 253:	return EL_UNKNOWN_EL;   	/* !!! Key Brown !!! */
    case 254:	return EL_UNKNOWN_EL;   	/* !!! Dirt 2 !!! */
    case 255:	return EL_UNKNOWN_EL;   	/* !!! Poison Dirt !!! */

#if 0
    case 221:	return EL_MOLE_LEFT;	   	/* mole left (DC) */
    case 222:	return EL_MOLE_UP;   		/* mole up (DC) */
    case 223:	return EL_MOLE_RIGHT;   	/* mole right (DC) */
    case 224:	return EL_MOLE_DOWN;   		/* mole down (DC) */
#endif

    default:	return EL_UNKNOWN_EL;
  }
}

void ConvertDXBoulderLevelFile(char *filename_old, char *level_dir)
{
  FILE *file;
  char filename_new[1024];
  char magic[5];
  int level_nr;
  int player_x, player_y;
  int red_gem_score;
  int string_length;
  int i, x, y;

  printf("Reading DXBoulder level file '%s'...\n", filename_old);

  if (!(file = fopen(filename_old, "r")))
  {
    fprintf(stderr, "%s: cannot open DXBoulder level file '%s'!\n",
	    progname, filename_old);
    exit(1);
  }

  /* read magic bytes at the start of the DXBoulder map header */
  for (i=0; i<4; i++)
    magic[i] = fgetc(file);
  magic[4] = '\0';

  if (strcmp(magic, "Stef") != 0)
  {
    fprintf(stderr, "%s: '%s' doesn't look like a DXBoulder level file!\n",
	    progname, filename_old);
    exit(1);
  }

  /* skip the (unused) rest of the DXBoulder map header */
  skip_bytes(file, DX_MAP_HEADER_SIZE - 4);

  /* now read the levels */
  for (level_nr=0; level_nr<NUM_DXBOULDER_LEVELS; level_nr++)
  {
    printf("|  Converting level '%d'\n", level_nr);

    lev_fieldx = level.fieldx = fgetc(file);
    lev_fieldy = level.fieldy = fgetc(file);

    player_x = fgetc(file);
    player_y = fgetc(file);
    level.gems_needed = fgetc(file);

    level.num_yam_contents = 4;
    for(i=0; i<8; i++)
    {
      for(x=0; x<3; x++)
      {
	for(y=0; y<3; y++)
	{
	  int element_old = fgetc(file);
	  int element_new = map_dx_element(element_old);

	  if (element_new == EL_UNKNOWN_EL)
	  {
	    printf("----------> Level %d: Oops! [%d][%d][%d]: element %d unknown! (YamYam)\n",
		   level_nr, i, x, y, element_old);

	    element_new = EL_CHAR_FRAGE;
	  }

	  level.yam_content[i][x][y] = element_new;
	}
      }
    }

    level.time = fgetc(file) * 10;
    level.time_magic_wall   = fgetc(file) | (fgetc(file) << 8); /* little endian */
    level.time_wheel = fgetc(file) | (fgetc(file) << 8); /* little endian */
    level.amoeba_speed = fgetc(file) | (fgetc(file) << 8); /* little endian */

    level.score[SC_EDELSTEIN] = fgetc(file);
    level.score[SC_DIAMANT] = fgetc(file);
    red_gem_score = fgetc(file);
    level.score[SC_DYNAMIT] = fgetc(file);
    level.score[SC_KOKOSNUSS] = fgetc(file);
    level.score[SC_ROBOT] = fgetc(file);
    level.score[SC_FLIEGER] = fgetc(file);
    level.score[SC_MAMPFER] = fgetc(file);
    level.score[SC_ZEITBONUS] = fgetc(file);
    level.score[SC_KAEFER] = fgetc(file);
    level.score[SC_SCHLUESSEL] = fgetc(file);

    string_length = fgetc(file);
    for(i=0; i<20; i++)
    {
      int c = fgetc(file);

      /* work around a little bug (non-printable character) in some levels */
      if (c == 0233)	/* 155 */
	c = ' ';	/* replace non-printable character with space */
      level.name[i] = c;
    }
    for(i=string_length; i<MAX_LEVEL_NAME_LEN; i++)
      level.name[i] = '\0';

    /* cut trailing spaces in level name */
    for (i=MAX_LEVEL_NAME_LEN; i>=0; i--)
    {
      int c = level.name[i];

      if (c != ' ' && c != '\0')
	break;
      else if (c == ' ')
	level.name[i] = '\0';
    }

    string_length = fgetc(file);
    for(i=0; i<20; i++)
      level.author[i] = fgetc(file);
    for(i=string_length; i<MAX_LEVEL_AUTHOR_LEN; i++)
      level.author[i] = '\0';

    /* skip level edit password */
    skip_bytes(file, 1 + 10);

    /* skip the (unused) rest of the level header */
    skip_bytes(file, 210);

    /* read level field data */
    for (y=0; y<lev_fieldy; y++)
    {
      for (x=0; x<lev_fieldx; x++)
      {
	int element_old = fgetc(file);
	int element_new = map_dx_element(element_old);

	if (element_new == EL_UNKNOWN_EL)
	{
	  printf("----------> Level %d: Oops! [%d][%d]: element %d unknown!\n",
		 level_nr, x, y, element_old);

	  element_new = EL_CHAR_FRAGE;
	}

	Ur[x][y] = element_new;
      }
    }

    level.amoeba_content = EL_LEERRAUM;
    level.double_speed = FALSE;         /* player has same speed as enemies */
    level.gravity = FALSE;

#if 0
    printf("level size: %d x %d\n", lev_fieldx, lev_fieldy);
    printf("level name: %s\n", level.name);
#endif

    /* save new level */
    sprintf(filename_new, "%s/%03d.level", level_dir, level_nr);
    printf("|  Writing new level file '%s'...\n", filename_new);

    SaveLevel(filename_new);
  }

  fclose(file);
}

void main(int argc, char *argv[])
{
  progname = argv[0];

  if (argc != 3)
  {
    printf("Usage: %s <old level file> <new level directory>\n", progname);
    exit(0);
  }

  ConvertDXBoulderLevelFile(argv[1], argv[2]);
}
