#include "main-1.1b.h"

void LoadOldLevel(char *);
void SaveNewLevel(int);

struct LevelInfo level;
int Ur[MAX_LEV_FIELDX][MAX_LEV_FIELDY];

char *progname;
char *levelfile;

int main(int argc, char *argv[])
{
  int i;

  progname = argv[0];
  levelfile = argv[1];

  if (argc != 2)
  {
    printf("Usage: %s <emerald mine level file>\n",progname);
    exit(0);
  }

  SaveNewLevel(LoadEmeraldMineLevel(levelfile));
}

int LoadEmeraldMineLevel(char *filename)
{
  int i,x,y;
  FILE *file;
  int level_nr, JX,JY;
  unsigned char body[40][64];
  unsigned char *leveldata = &body[0][0];
  unsigned char *header = &leveldata[2048];
  unsigned char code0 = 0x65;
  unsigned char code1 = 0x11;

  printf("Reading '%s'\n",levelfile);

  if (!(file=fopen(levelfile,"r")))
  {
    fprintf(stderr,"%s: cannot load level %s!\n",
	    progname,filename);
    return(-1);
  }

  for(i=0;i<2106;i++)
    leveldata[i] = fgetc(file);
  fclose(file);

  for(i=0;i<2106;i++)
  {
    leveldata[i] ^= code0;
    leveldata[i] -= code1;
    code0  = (code0 + 7) % 256;
  }

  level.fieldx			= 64;
  level.fieldy			= 32;
  level.time			= header[46]*10;
  level.edelsteine		= header[47];

  level_nr = leveldata[1];
  leveldata[1] = leveldata[2];

  for(i=0;i<MAX_LEVNAMLEN;i++)
    level.name[i] = 0;
  sprintf(level.name,"Emerald Mine Level %d",level_nr);

  for(i=0;i<MAX_LEVSCORE_ENTRIES;i++)
    level.score[i]		= 10;
  level.score[SC_EDELSTEIN]	= header[36];
  level.score[SC_DIAMANT]	= header[37];
  level.score[SC_ROBOT]		= header[38];
  level.score[SC_FLIEGER]	= header[39];
  level.score[SC_KAEFER]	= header[40];
  level.score[SC_MAMPFER]	= header[41];
  level.score[SC_KOKOSNUSS]	= header[42];
  level.score[SC_DYNAMIT]	= header[43];
  level.score[SC_ZEITBONUS]	= header[44];

  JX = header[48]*256+header[49] % 64;
  JY = header[48]*256+header[49] / 64;

  for(i=0;i<4;i++) for(y=0;y<3;y++) for(x=0;x<3;x++)
  {
    int element = header[i*9+y*3+x];
    int new_element;

    new_element = (element==0x00 ? EL_LEERRAUM :
		   element==0x01 ? EL_EDELSTEIN :
		   element==0x02 ? EL_DIAMANT :
		   element==0x03 ? EL_FELSBROCKEN :
		   element==0x04 ? EL_ROBOT :
		   element==0x05 ? EL_FLIEGER_O :
		   element==0x06 ? EL_BOMBE :
		   element==0x07 ? EL_KAEFER_O :
		   element==0x08 ? EL_TROPFEN :
		   element==0x09 ? EL_KOKOSNUSS :
		   element==0x0a ? EL_MAMPFER :
		   element==0x0b ? EL_MORAST_VOLL :
		   element==0x0c ? EL_ERDREICH :
		   element==0x0d ? EL_FELSBODEN :
		   element==0x0e ? EL_BETON :
		   element==0x0f ? EL_MAUERWERK :
		   element==0x10 ? EL_EM_KEY_1_FILE :
		   element==0x11 ? EL_EM_KEY_2_FILE :
		   element==0x12 ? EL_EM_KEY_4_FILE :
		   element==0x13 ? EL_EM_KEY_3_FILE :
		   element==0x14 ? EL_SIEB_LEER :
		   element==0x15 ? EL_ABLENK_AUS :
		   element==0x16 ? EL_DYNAMIT_AUS : -1);

    if (new_element==-1)
    {
      printf("  Hopsa! [%d][%d][%d]: Element 0x%02x nicht bekannt!\n",
	     i,x,y,element);
      new_element = EL_LEERRAUM;
    }

    level.mampfer_inhalt[i][x][y] = new_element;
  }

/*
  printf("Zeit:   %ld\n",header[46]);
  printf("Ambe:  %ld\n",header[52]*256+header[53]);
  printf("Sieb:   %ld\n",header[54]*256+header[55]);
  printf("Ablenk: %ld\n",header[56]*256+header[57]);
*/

  level.tempo_amoebe		= (header[52]*256+header[53]) % 256;
  level.dauer_sieb		= (header[54]*256+header[55]) * 16 / 100;
  level.dauer_ablenk		= (header[56]*256+header[57]) * 16 / 100;
  level.amoebe_inhalt		= EL_DIAMANT;

  for(y=0;y<32;y++) for(x=0;x<64;x++)
  {
    int element = body[y][x];
    int new_element;

    new_element = (element==0x00 ? EL_FELSBROCKEN :
		   element==0x02 ? EL_DIAMANT :
		   element==0x03 ? EL_DIAMANT :
		   element==0x04 ? EL_ROBOT :
		   element==0x08 ? EL_FLIEGER_O :
		   element==0x09 ? EL_FLIEGER_R :
		   element==0x0a ? EL_FLIEGER_U :
		   element==0x0b ? EL_FLIEGER_L :
		   element==0x0c ? EL_FLIEGER_O :
		   element==0x0d ? EL_FLIEGER_R :
		   element==0x0e ? EL_FLIEGER_U :
		   element==0x0f ? EL_FLIEGER_L :
		   element==0x10 ? EL_BOMBE :
		   element==0x12 ? EL_EDELSTEIN :
		   element==0x13 ? EL_EDELSTEIN :
		   element==0x14 ? EL_KAEFER_O :
		   element==0x15 ? EL_KAEFER_R :
		   element==0x16 ? EL_KAEFER_U :
		   element==0x17 ? EL_KAEFER_L :
		   element==0x18 ? EL_KAEFER_O :
		   element==0x19 ? EL_KAEFER_R :
		   element==0x1a ? EL_KAEFER_U :
		   element==0x1b ? EL_KAEFER_L :
		   element==0x1c ? EL_TROPFEN :
		   element==0x20 ? EL_FELSBROCKEN :
		   element==0x24 ? EL_SIEB_LEER :
		   element==0x25 ? EL_KOKOSNUSS :
		   element==0x29 ? EL_MAMPFER :
		   element==0x2a ? EL_MAMPFER :
		   element==0x2d ? EL_MORAST_VOLL :
		   element==0x3b ? EL_DYNAMIT :
		   element==0x3c ? EL_DYNAMIT :
		   element==0x3d ? EL_DYNAMIT :
		   element==0x3e ? EL_DYNAMIT :
		   element==0x3f ? EL_BADEWANNE4 :
		   element==0x40 ? EL_AUSGANG_AUF :
		   element==0x4e ? EL_UNSICHTBAR :
		   element==0x74 ? EL_BETON :
		   element==0x7b ? EL_SALZSAEURE :
		   element==0x80 ? EL_LEERRAUM :
		   element==0x81 ? EL_FELSBODEN :
		   element==0x82 ? EL_ERDREICH :
		   element==0x83 ? EL_BETON :
		   element==0x84 ? EL_MAUERWERK :
		   element==0x85 ? EL_EM_KEY_1_FILE :
		   element==0x86 ? EL_EM_KEY_2_FILE :
		   element==0x87 ? EL_EM_KEY_4_FILE :
		   element==0x88 ? EL_EM_KEY_3_FILE :
		   element==0x89 ? EL_EM_GATE_1 :
		   element==0x8a ? EL_EM_GATE_2 :
		   element==0x8b ? EL_EM_GATE_4 :
		   element==0x8c ? EL_EM_GATE_3 :
		   element==0x8e ? EL_EM_GATE_1X :
		   element==0x8f ? EL_EM_GATE_2X :
		   element==0x90 ? EL_EM_GATE_4X :
		   element==0x91 ? EL_EM_GATE_3X :
		   element==0x92 ? EL_SIEB_LEER :
		   element==0x94 ? EL_MORAST_LEER :
		   element==0x95 ? EL_BADEWANNE1 :
		   element==0x96 ? EL_BADEWANNE2 :
		   element==0x97 ? EL_BADEWANNE3 :
		   element==0x98 ? EL_BADEWANNE5 :
		   element==0x99 ? EL_SALZSAEURE :
		   element==0x9a ? EL_AMOEBE_TOT :
		   element==0x9b ? EL_AMOEBE_TOT :
		   element==0x9c ? EL_AMOEBE_TOT :
		   element==0x9d ? EL_AMOEBE_TOT :
		   element==0x9e ? EL_AUSGANG_ZU :
		   element==0x93 ? EL_ABLENK_AUS :
		   element==0xaf ? EL_DYNAMIT_AUS : -1);

    if (new_element==EL_AMOEBE_TOT && level.tempo_amoebe)
      new_element = EL_AMOEBE_NASS;
    else if (new_element==-1)
    {
      if (element>=0xc8 && element<=0xe1)	/* Buchstaben A bis Z */
	new_element = EL_CHAR_A + (element - 0xc8);
      else if (element>=0xe2 && element<=0xeb)	/* Zahlen 0 bis 9 */
	new_element = EL_CHAR_0 + (element - 0xe2);
      else if (element==0xec)			/* Punkt */
	new_element = EL_CHAR_PUNKT;
      else if (element==0xed)			/* Ausrufezeichen */
	new_element = EL_CHAR_AUSRUF;
      else if (element==0xee)			/* Doppelpunkt */
	new_element = EL_CHAR_DOPPEL;
      else if (element==0xef)			/* Fragezeichen */
	new_element = EL_CHAR_FRAGE;
      else if (element==0xf0)			/* Pfeil nach rechts */
	new_element = EL_CHAR_FRAGE;
      else if (element==0xf1)			/* Copyright */
	new_element = EL_CHAR_COPY;
      else if (element==0xfe)			/* Spieler 1 */
	new_element = EL_SPIELFIGUR;
      else if (element==0xff)			/* Spieler 2 */
	new_element = EL_LEERRAUM;
      else
      {
	printf("  Hopsa! [%d][%d]: Element 0x%02x nicht bekannt!\n",
	       x,y,element);
	new_element = EL_LEERRAUM;
      }
    }

    Ur[x][y] = new_element;
  }

  JX = (header[48]*256+header[49]) % 64;
  JY = (header[48]*256+header[49]) / 64;
  Ur[JX][JY] = EL_SPIELER1;

  JX = (header[50]*256+header[51]) % 64;
  JY = (header[50]*256+header[51]) / 64;
  Ur[JX][JY] = EL_SPIELER2;

  return(level_nr);
}

void SaveNewLevel(int level_nr)
{
  int i,x,y;
  char filename[256];
  FILE *file;

  sprintf(filename,"%s/%03d.level",LEVEL_PATH,level_nr);

  printf("Writing '%s'\n",filename);

  if (!(file=fopen(filename,"w")))
  {
    fprintf(stderr,"%s: cannot save level %d!\n",
	    progname,level_nr);
    return;
  }

  fprintf(file,"%s\n",LEVEL_COOKIE);	/* Formatkennung */

  fputc(level.fieldx,file);
  fputc(level.fieldy,file);
  fputc(level.time / 256,file);
  fputc(level.time % 256,file);
  fputc(level.edelsteine / 256,file);
  fputc(level.edelsteine % 256,file);

  for(i=0;i<MAX_LEVNAMLEN;i++)
    fputc(level.name[i],file);
  for(i=0;i<MAX_LEVSCORE_ENTRIES;i++)
    fputc(level.score[i],file);
  for(i=0;i<4;i++)
    for(y=0;y<3;y++)
      for(x=0;x<3;x++)
	fputc(level.mampfer_inhalt[i][x][y],file);
  fputc(level.tempo_amoebe,file);
  fputc(level.dauer_sieb,file);
  fputc(level.dauer_ablenk,file);
  fputc(level.amoebe_inhalt,file);

  for(i=0;i<NUM_FREE_LVHD_BYTES;i++)	/* Rest frei / Headergre 80 Bytes */
    fputc(0,file);

  for(y=0;y<32;y++) 
    for(x=0;x<64;x++) 
      fputc(Ur[x][y],file);

  fclose(file);
}
