#include "main-1.1b.h"

#define NUM_SOKOBAN_LEVELS	50

void LoadSokobanLevel(char *, int);
void SaveNewLevel(char *);
void FloodFill(int, int, int);

struct LevelInfo level;
int Ur[MAX_LEV_FIELDX][MAX_LEV_FIELDY];

char *progname;
char *levelfile_old;
char *levelfile_new;

void main(int argc, char *argv[])
{
  int levelnr;

  progname = argv[0];

  if (argc != 3)
  {
    printf("Usage: %s <sokoban directory> <new level directory>\n", progname);
    exit(0);
  }

  for(levelnr=1;levelnr<=NUM_SOKOBAN_LEVELS;levelnr++)
  {
    char levelfile_old[50];
    char levelfile_new[50];

    sprintf(levelfile_old, "%s/screens/screen.%d", argv[1], levelnr);
    sprintf(levelfile_new, "%s/%03d.level", argv[2], levelnr);

    LoadSokobanLevel(levelfile_old, levelnr);
    SaveNewLevel(levelfile_new);
  }
}

void LoadSokobanLevel(char *filename, int levelnr)
{
  int i,x,y;
  FILE *file;
  char c;
  BOOL spieler = FALSE;
  int real_xsize = 1, real_ysize = 1;
  int player_x, player_y;

  printf("Decoding Sokoban level %d\n", levelnr);

  if (!(file=fopen(filename,"r")))
  {
    fprintf(stderr,"%s: cannot load level %s!\n",
	    progname,filename);
    exit(0);
  }

  level.fieldx = STD_LEV_FIELDX;
  level.fieldy = STD_LEV_FIELDY;

  level.time		= 999;
  level.edelsteine	= 0;
  for(i=0;i<MAX_LEVNAMLEN;i++)
    level.name[i] = 0;
  sprintf(level.name,"Sokoban Level %d",levelnr);
  for(i=0;i<MAX_LEVSCORE_ENTRIES;i++)
    level.score[i]	= 10;
  for(i=0;i<4;i++)
    for(y=0;y<3;y++)
      for(x=0;x<3;x++)
	level.mampfer_inhalt[i][x][y] = EL_FELSBROCKEN;
  level.tempo_amoebe	= 10;
  level.dauer_sieb	= 10;
  level.dauer_ablenk	= 10;
  level.amoebe_inhalt = EL_EDELSTEIN_BD;

  for(y=0;y<STD_LEV_FIELDY;y++) 
    for(x=0;x<STD_LEV_FIELDX;x++) 
      Ur[x][y] = EL_LEERRAUM;

/*
  Ur[0][0] = EL_SPIELFIGUR;
  Ur[STD_LEV_FIELDX-1][STD_LEV_FIELDY-1] = EL_AUSGANG_ZU;
*/

  x = y = 0;

  do
  {
    c = fgetc(file);
    if (c == '\n')
    {
      x = 0;
      y++;
    }
    else
    {
      Ur[x+1][y+1] = (c==' ' ? EL_LEERRAUM :
		      c=='#' ? EL_BETON :
		      c=='$' ? EL_SOKOBAN_OBJEKT :
		      c=='.' ? EL_SOKOBAN_FELD_LEER :
		      c=='*' ? EL_SOKOBAN_FELD_VOLL :
		      c=='@' ? EL_SPIELFIGUR : EL_LEERRAUM);

      if (c == '@')
      {
	spieler = TRUE;
	player_x = x;
	player_y = y;
      }

      x++;
      real_xsize = MAX(real_xsize, x);
      real_ysize = MAX(real_ysize, y);
    }
  }
  while(!feof(file));

  fclose(file);

#if 0
  FloodFill(0,0,EL_BETON);
#else
  FloodFill(0,0,EL_INVISIBLE_STEEL);
#endif

  level.fieldx = real_xsize;
  level.fieldy = real_ysize;

  for(y=0;y<level.fieldy;y++) 
    for(x=0;x<level.fieldx;x++) 
      Ur[x][y] = Ur[x+1][y+1];

  /* no exit needed for sokoban levels! */
/*
  for(x=player_x,y=player_y;y<level.fieldy;y++) 
  {
    if (Ur[x][y] == EL_BETON)
    {
      Ur[x][y] = EL_AUSGANG_ZU;
      break;
    }
  }
*/

  if (!spieler)
  {
    printf("No player in level %d!\n", levelnr);
    exit(0);
  }
}

void SaveNewLevel(char *filename)
{
  int i,x,y;
  FILE *file;

  printf("Writing '%s'\n",filename);

  if (!(file=fopen(filename,"w")))
  {
    fprintf(stderr,"%s: cannot save level file '%s'!\n",progname,filename);
    return;
  }

  fputs(LEVEL_COOKIE,file);		/* Formatkennung */
  fputc(0x0a,file);

  fputc(level.fieldx,file);
  fputc(level.fieldy,file);
  fputc(level.time / 256,file);
  fputc(level.time % 256,file);
  fputc(level.edelsteine / 256,file);
  fputc(level.edelsteine % 256,file);

  for(i=0;i<MAX_LEVNAMLEN;i++)
    fputc(level.name[i],file);
  for(i=0;i<MAX_LEVSCORE_ENTRIES;i++)
    fputc(level.score[i],file);
  for(i=0;i<4;i++)
    for(y=0;y<3;y++)
      for(x=0;x<3;x++)
	fputc(level.mampfer_inhalt[i][x][y],file);
  fputc(level.tempo_amoebe,file);
  fputc(level.dauer_sieb,file);
  fputc(level.dauer_ablenk,file);
  fputc(level.amoebe_inhalt,file);

  for(i=0;i<NUM_FREE_LVHD_BYTES;i++)	/* Rest frei / Headergre 80 Bytes */
    fputc(0,file);

  for(y=0;y<level.fieldy;y++) 
    for(x=0;x<level.fieldx;x++) 
      fputc(Ur[x][y],file);

  fclose(file);

  chmod(filename, LEVEL_PERMS);
}

#define IN_SOKOBAN_FIELD(x,y) ((x)>=0 && (x)<level.fieldx && (y)>=0 &&(y)<level.fieldy)

void FloodFill(int from_x, int from_y, int fill_element)
{
  int i,x,y;
  int old_element;
  static int check[4][2] = { {-1,0}, {0,-1}, {1,0}, {0,1} };
  static int safety = 0;

  safety++;

  if (safety>level.fieldx*level.fieldy)
  {
    fprintf(stderr,"Something went wrong in 'FloodFill()'. Please debug.\n");
    exit(-1);
  }

  old_element = Ur[from_x][from_y];
  Ur[from_x][from_y] = fill_element;

  for(i=0;i<4;i++)
  {
    x = from_x+check[i][0];
    y = from_y+check[i][1];

    if (IN_SOKOBAN_FIELD(x,y) && Ur[x][y]==old_element)
      FloodFill(x,y,fill_element);
  }

  safety--;
}
