
#include "../src/main.h"

#define NUM_SUPAPLEX_LEVELS	111
#define SP_LEVEL_SIZE		1536
#define SP_LEVEL_XSIZE		60
#define SP_LEVEL_YSIZE		24
#define SP_LEVEL_NAME_LEN	23

#define LEVEL_HEADER_SIZE	80	/* size of level file header */
#define LEVEL_HEADER_UNUSED	16	/* unused level header bytes */

/* file identifier strings */
#define LEVEL_COOKIE		"ROCKSNDIAMONDS_LEVEL_FILE_VERSION_1.2"

/* file permissions for newly written files */
#define MODE_R_ALL		(S_IRUSR | S_IRGRP | S_IROTH)
#define MODE_W_ALL		(S_IWUSR | S_IWGRP | S_IWOTH)
#define MODE_X_ALL		(S_IXUSR | S_IXGRP | S_IXOTH)
#define LEVEL_PERMS		(MODE_R_ALL | MODE_W_ALL)

char *progname;

struct LevelInfo level;
short Ur[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
int MampferMax;
int lev_fieldx, lev_fieldy;

static void putFileChunk(FILE *file, char *chunk_name, int chunk_length)
{
  fputs(chunk_name, file);

  fputc((chunk_length >> 24) & 0xff, file);
  fputc((chunk_length >> 16) & 0xff, file);
  fputc((chunk_length >>  8) & 0xff, file);
  fputc((chunk_length >>  0) & 0xff, file);
}

void SaveLevel(char *filename)
{
  int i, x, y;
  FILE *file;

  if (!(file = fopen(filename, "w")))
  {
    fprintf(stderr, "%s: cannot save level file '%s'", progname, filename);
    exit(1);
  }

  fputs(LEVEL_COOKIE, file);		/* file identifier */
  fputc('\n', file);

  putFileChunk(file, "HEAD", LEVEL_HEADER_SIZE);

  fputc(level.fieldx, file);
  fputc(level.fieldy, file);
  fputc(level.time / 256, file);
  fputc(level.time % 256, file);
  fputc(level.gems_needed / 256, file);
  fputc(level.gems_needed % 256, file);

  for(i=0; i<MAX_LEVEL_NAME_LEN; i++)
    fputc(level.name[i], file);
  for(i=0; i<LEVEL_SCORE_ELEMENTS; i++)
    fputc(level.score[i], file);
  for(i=0; i<4; i++)
    for(y=0; y<3; y++)
      for(x=0; x<3; x++)
	fputc(level.yam_content[i][x][y], file);
  fputc(level.amoeba_speed, file);
  fputc(level.time_magic_wall, file);
  fputc(level.time_wheel, file);
  fputc(level.amoeba_content, file);
  fputc((level.double_speed ? 1 : 0), file);
  fputc((level.gravity ? 1 : 0), file);

  for(i=0; i<LEVEL_HEADER_UNUSED; i++)	/* set unused header bytes to zero */
    fputc(0, file);

  /* don't set author field -- will be overridden by 'levelinfo.conf' */
  /*
  putFileChunk(file, "AUTH", MAX_LEVEL_AUTHOR_LEN);

  for(i=0; i<MAX_LEVEL_AUTHOR_LEN; i++)
    fputc(level.author[i], file);
  */

  /*
  putFileChunk(file, "CONT", 4 + 8 * 3 * 3);

  fputc(EL_MAMPFER, file);
  fputc(MampferMax, file);
  fputc(0, file);
  fputc(0, file);

  for(i=0; i<8; i++)
    for(y=0; y<3; y++)
      for(x=0; x<3; x++)
	fputc(level.yam_content[i][x][y], file);
  */

  putFileChunk(file, "BODY", lev_fieldx * lev_fieldy);

  for(y=0; y<lev_fieldy; y++) 
    for(x=0; x<lev_fieldx; x++) 
      fputc(Ur[x][y], file);

  fclose(file);

  chmod(filename, LEVEL_PERMS);
}

void SaveNewLevel(char *level_dir, int level_nr)
{
  char filename_new[1024];

  sprintf(filename_new, "%s/%03d.level", level_dir, level_nr);
  printf("Writing new level file '%s'...\n", filename_new);
  SaveLevel(filename_new);
}

/* global values for multipart levels */
boolean is_multipart_level = FALSE;
struct LevelInfo multipart_level;
short multipart_Ur[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
char first_char, last_char;
int multipart_x, multipart_y;

void SaveNewMultipartLevel(char *level_dir, int level_nr)
{
  int x, y;
  struct LevelInfo backup_level;
  int backup_lev_fieldx, backup_lev_fieldy;
  short backup_Ur[MAX_LEV_FIELDX][MAX_LEV_FIELDY];

  /* backup global values */
  backup_level = level;
  backup_lev_fieldx = lev_fieldx;
  backup_lev_fieldy = lev_fieldy;
  for (y=0; y<SP_LEVEL_YSIZE; y++)
    for (x=0; x<SP_LEVEL_XSIZE; x++)
      backup_Ur[x][y] = Ur[x][y];

  level = multipart_level;
  lev_fieldx = level.fieldx;
  lev_fieldy = level.fieldy;

  for (y=0; y<MAX_LEV_FIELDY; y++)
    for (x=0; x<MAX_LEV_FIELDX; x++)
      Ur[x][y] = multipart_Ur[x][y];

  printf("Writing multipart level %d: '%s' (%d x %d)\n  ",
	 level_nr, level.name,
	 level.fieldx / SP_LEVEL_XSIZE, level.fieldy / SP_LEVEL_YSIZE);
  SaveNewLevel(level_dir, level_nr);

  /* restore global values */
  level = backup_level;
  lev_fieldx = backup_lev_fieldx;
  lev_fieldy = backup_lev_fieldy;
  for (y=0; y<SP_LEVEL_YSIZE; y++)
    for (x=0; x<SP_LEVEL_XSIZE; x++)
      Ur[x][y] = backup_Ur[x][y];
}

void ConvertSupaplexLevelFile(char *filename_old, char *level_dir)
{
  FILE *file;
  char level_name[SP_LEVEL_NAME_LEN + 1];
  int skip_bytes, dummy;
  int level_nr;
  int i, x, y;
  boolean new_multipart;

  printf("Converting supaplex level file '%s'...\n", filename_old);

  if (!(file = fopen(filename_old, "r")))
  {
    fprintf(stderr, "%s: cannot open supaplex level file '%s'!\n",
	    progname, filename_old);
    exit(1);
  }

  for (level_nr=1; level_nr<=NUM_SUPAPLEX_LEVELS; level_nr++)
  {
    /* The level itself (width * height == 60 * 24 == 1440 bytes) */
    for (y=0; y<SP_LEVEL_YSIZE; y++)
    {
      for (x=0; x<SP_LEVEL_XSIZE; x++)
      {
	int element_old = fgetc(file);
	int element_new;

	if (element_old <= 0x27)
	  element_new = EL_SP_START + element_old;
	else if (element_old == 0x28)
	  element_new = EL_UNSICHTBAR;
	else
	{
	  printf("----------> Level %d: Oops! [%d][%d]: element 0x%02x unknown!\n",
		 level_nr, x, y, element_old);

	  element_new = EL_CHAR_FRAGE;
	}

	Ur[x][y] = element_new;
      }
    }

    /* skip 4 unused bytes */
    skip_bytes = 4;
    for (i=0; i<skip_bytes; i++)
      fgetc(file);

    /* Initial gravitation: 1=on, anything else (0) = off */
    level.gravity = (fgetc(file) == 1 ? TRUE : FALSE);

#if 0
    if (level.gravity)
      printf("INFO: Level %d has gravity switched on.\n", level_nr);
#endif

    /* skip 1 unused byte */
    dummy = fgetc(file);

    /* Level title in uppercase ASCII, stuffed with dashes ("-") (23 bytes) */
    for (i=0; i<SP_LEVEL_NAME_LEN; i++)
      level_name[i] = fgetc(file);
    level_name[SP_LEVEL_NAME_LEN] = '\0';

    printf("Name of level %d is '%s'\n", level_nr, level_name);

    /* Initial "Freeze Zonks": 2=on, anything else (0) = off.  (1=off too!) */
    /* NOT SUPPORTED YET */
    dummy = fgetc(file);

    /* Number of Infotrons needed. 0 means that Supaplex will count the total
       amount of Infotrons in the level, and use the low byte of that number.
       (A multiple of 256 Infotrons will then result in 0-to-eat, etc.!) */
    level.gems_needed = fgetc(file);

    skip_bytes = SP_LEVEL_SIZE;
    skip_bytes -= SP_LEVEL_XSIZE * SP_LEVEL_YSIZE;
    skip_bytes -= 6;
    skip_bytes -= SP_LEVEL_NAME_LEN;
    skip_bytes -= 2;

    /* information about special ("gravity") port entries */
    /* NOT SUPPORTED YET */
    for (i=0; i<skip_bytes; i++)
      fgetc(file);

    lev_fieldx = level.fieldx = SP_LEVEL_XSIZE;
    lev_fieldy = level.fieldy = SP_LEVEL_YSIZE;

    level.time = 0;			/* no time limit */
    level.amoeba_speed = 0;
    level.time_magic_wall = 0;
    level.time_wheel = 0;
    level.amoeba_content = EL_LEERRAUM;

    level.double_speed = FALSE;		/* player has same speed as enemies */

    /* set level name string to all zeroes */
    for(i=0; i<MAX_LEVEL_NAME_LEN; i++)
      level.name[i] = 0;

    /* copy level name */
    strcpy(level.name, level_name);

    for(i=0; i<LEVEL_SCORE_ELEMENTS; i++)
      level.score[i] = 0;

    /* there are no crunchers in supaplex levels */
    MampferMax = 4;
    for(i=0; i<8; i++)
      for(x=0; x<3; x++)
	for(y=0; y<3; y++)
	  level.yam_content[i][x][y] = EL_LEERRAUM;

    /* --------------------------------------------------------------------- */

    /* check for multipart level */
    first_char = level_name[0];
    last_char = level_name[SP_LEVEL_NAME_LEN - 1];

    new_multipart =
      ((first_char == '?' || (first_char >= '0' && first_char <= '9')) &&
       (last_char  == '?' || (last_char  >= '0' && last_char  <= '9')));

    if (!new_multipart)
    {
      if (is_multipart_level)
      {
	/* we still have a multipart level, but this is a single part level */
	SaveNewMultipartLevel(level_dir, level_nr - 1);
	is_multipart_level = FALSE;
      }
    }
    else	/* this is a new (or another) part of a multipart level */
    {
      /* replace leading multipart info chars */
      for (i=0; i<SP_LEVEL_NAME_LEN; i++)
      {
	if (level.name[i] == first_char)
	  level.name[i] = '-';
	else
	  break;
      }

      /* replace trailing multipart info chars */
      for (i=SP_LEVEL_NAME_LEN - 1; i>=0; i--)
      {
	if (level.name[i] == last_char)
	  level.name[i] = '-';
	else
	  break;
      }

      if (is_multipart_level && strcmp(level.name, multipart_level.name) != 0)
      {
	/* we still have a multipart level, but here starts a new one */
	SaveNewMultipartLevel(level_dir, level_nr - 1);
	is_multipart_level = FALSE;
      }

      if (!is_multipart_level)
      {
	/* this is a new multipart level */
	for (y=0; y<MAX_LEV_FIELDY; y++)
	  for (x=0; x<MAX_LEV_FIELDX; x++)
	    multipart_Ur[x][y] = EL_SP_EMPTY;

	/* copy LevelInfo structure */
	multipart_level = level;

	is_multipart_level = TRUE;
      }

      if (first_char == '?')
	first_char = '1';
      if (last_char == '?')
	last_char = '1';

      multipart_x = (int)(first_char - '0');
      multipart_y = (int)(last_char - '0');

      printf("----------> Part (%d/%d) of multipart level '%s'\n",
	     multipart_x, multipart_y, multipart_level.name);

      if (multipart_x * SP_LEVEL_XSIZE > MAX_LEV_FIELDX ||
	  multipart_y * SP_LEVEL_YSIZE > MAX_LEV_FIELDY)
      {
	printf("ERROR: Multi-part level is too big! Ignoring this part\n");
      }
      else
      {
	multipart_level.fieldx = MAX(multipart_level.fieldx,
				     multipart_x * SP_LEVEL_XSIZE);
	multipart_level.fieldy = MAX(multipart_level.fieldy,
				     multipart_y * SP_LEVEL_YSIZE);

	/* copy level part at the right position of multipart level */
	for (y=0; y<SP_LEVEL_YSIZE; y++)
	{
	  for (x=0; x<SP_LEVEL_XSIZE; x++)
	  {
	    int start_x = (multipart_x - 1) * SP_LEVEL_XSIZE;
	    int start_y = (multipart_y - 1) * SP_LEVEL_YSIZE;

	    multipart_Ur[start_x + x][start_y + y] = Ur[x][y];
	  }
	}
      }

      /* writing empty level instead of single part of multipart level */
      for (y=0; y<SP_LEVEL_YSIZE; y++)
	for (x=0; x<SP_LEVEL_XSIZE; x++)
	  Ur[x][y] = EL_SP_EMPTY;

      strncpy(level.name, "-------- EMPTY --------", SP_LEVEL_NAME_LEN);
      level.name[SP_LEVEL_NAME_LEN] = '\0';

      level.gems_needed = 0;

      printf("Writing empty dummy level\n  ");
    }

    /* --------------------------------------------------------------------- */

    SaveNewLevel(level_dir, level_nr);
  }

  if (is_multipart_level)
  {
    /* we still have a multipart level after processing all input levels */
    SaveNewMultipartLevel(level_dir, level_nr - 1);
    is_multipart_level = FALSE;
  }

  fclose(file);
}

void main(int argc, char *argv[])
{
  progname = argv[0];

  if (argc != 3)
  {
    printf("Usage: %s <old level file> <new level directory>\n", progname);
    exit(0);
  }

  ConvertSupaplexLevelFile(argv[1], argv[2]);
}
