#include "../src/main.h"

#define LEVEL_COOKIE            "ROCKSNDIAMONDS_LEVEL_FILE_VERSION_1.0"
#define NUM_FREE_LVHD_BYTES     18
#define LEVEL_PERMS     (S_IRUSR|S_IWUSR | S_IRGRP|S_IWGRP | S_IROTH|S_IWOTH)
#define SC_EDELSTEIN            0

void LoadXbdLevel(char *);
void SaveNewLevel(char *);

struct LevelInfo level;
short Ur[MAX_LEV_FIELDX][MAX_LEV_FIELDY];

char *progname;
char *levelfile_old;
char *levelfile_new;

void main(int argc, char *argv[])
{
  progname = argv[0];

  if (argc != 3)
  {
    printf("Usage: %s <xbd level file> <Rocks'n'Diamond level file>\n",
	   progname);
    exit(0);
  }

  levelfile_old = argv[1];
  levelfile_new = argv[2];

  LoadXbdLevel(levelfile_old);
  SaveNewLevel(levelfile_new);
}

void LoadXbdLevel(char *filename)
{
  int i,x,y;
  FILE *file;
  char name[1024];
  int dummy;
  char c;
  boolean spieler = FALSE;

  printf("Reading '%s'\n",filename);

  if (!(file=fopen(filename,"r")))
  {
    fprintf(stderr,"%s: cannot load level %s!\n",
	    progname,filename);
    exit(0);
  }

  level.fieldx = STD_LEV_FIELDX;
  level.fieldy = STD_LEV_FIELDY;

  level.time		= 100;
  level.gems_needed	= 0;
  for(i=0;i<MAX_LEVEL_NAME_LEN;i++)
    level.name[i] = 0;
  strncpy(level.name,"Nameless Xbd Level",MAX_LEVEL_NAME_LEN-1);
  for(i=0;i<LEVEL_SCORE_ELEMENTS;i++)
    level.score[i]	= 10;
  for(i=0;i<4;i++)
    for(y=0;y<3;y++)
      for(x=0;x<3;x++)
	level.yam_content[i][x][y] = EL_FELSBROCKEN;
  level.amoeba_speed	= 4;
  level.time_magic_wall	= 0;
  level.time_wheel	= 0;
  level.amoeba_content = EL_EDELSTEIN_BD;

  for(y=0;y<STD_LEV_FIELDY;y++) 
    for(x=0;x<STD_LEV_FIELDX;x++) 
      Ur[x][y] = EL_ERDREICH;
  Ur[0][0] = EL_SPIELFIGUR;
  Ur[STD_LEV_FIELDX-1][STD_LEV_FIELDY-1] = EL_AUSGANG_ZU;

  fscanf(file,"%d",&level.fieldy);
  fscanf(file,"%d",&level.fieldx);
  fscanf(file,"%d",&dummy);
  fscanf(file,"%d",&level.gems_needed);
  fscanf(file,"%d",&level.score[SC_EDELSTEIN]);
  fscanf(file,"%d",&dummy);
  fscanf(file,"%d",&level.time_magic_wall);
  level.time_magic_wall /= 10;
  fscanf(file,"%d",&dummy);
  fscanf(file,"%d",&level.time);
  level.time /= 10;
  fscanf(file,"%s",name);
  c = fgetc(file);

  for(i=0;i<MAX_LEVEL_NAME_LEN;i++)
  {
    if (name[i]=='_')
      level.name[i] = ' ';
    else
      level.name[i] = name[i];
  }
  level.name[MAX_LEVEL_NAME_LEN-1] = 0;

  for(y=0;y<level.fieldy;y++)
  {
    for(x=0;x<level.fieldx;x++)
    {
      c = fgetc(file);
      Ur[x][y] = (c==' ' ? EL_LEERRAUM :
		  c=='b' ? EL_FELSBROCKEN :
		  c=='d' ? EL_EDELSTEIN_BD :
		  c=='e' ? EL_MAMPFER2 :
		  c=='g' ? EL_ERDREICH :
		  c=='l' ? EL_FIREFLY_UP :
		  c=='p' ? EL_SPIELFIGUR :
		  c=='r' ? EL_BUTTERFLY_UP :
		  c=='t' ? EL_MAGIC_WALL_BD_OFF :
		  c=='w' ? EL_FELSBODEN :
		  c=='x' ? EL_DYNAMITE_ACTIVE :
		  c=='B' ? EL_AMOEBE_BD :
		  c=='E' ? EL_AUSGANG_ZU :
		  c=='N' ? EL_BOMBE :
		  c=='S' ? EL_BETON :
		  c=='W' ? EL_MAUER_LEBT : 0);

      if (c=='p')
	spieler = TRUE;
    }
    c = fgetc(file);
  }
  fclose(file);

  if (!spieler)
  {
    printf("%s: No player in this level!\n",
	   progname);
    exit(0);
  }
}

void SaveNewLevel(char *filename)
{
  int i,x,y;
  FILE *file;

  printf("Writing '%s'\n",filename);

  if (!(file=fopen(filename,"w")))
  {
    fprintf(stderr,"%s: cannot save level file '%s'!\n",progname,filename);
    return;
  }

  fputs(LEVEL_COOKIE,file);		/* Formatkennung */
  fputc(0x0a,file);

  fputc(level.fieldx,file);
  fputc(level.fieldy,file);
  fputc(level.time / 256,file);
  fputc(level.time % 256,file);
  fputc(level.gems_needed / 256,file);
  fputc(level.gems_needed % 256,file);

  for(i=0;i<MAX_LEVEL_NAME_LEN;i++)
    fputc(level.name[i],file);
  for(i=0;i<LEVEL_SCORE_ELEMENTS;i++)
    fputc(level.score[i],file);
  for(i=0;i<4;i++)
    for(y=0;y<3;y++)
      for(x=0;x<3;x++)
	fputc(level.yam_content[i][x][y],file);
  fputc(level.amoeba_speed,file);
  fputc(level.time_magic_wall,file);
  fputc(level.time_wheel,file);
  fputc(level.amoeba_content,file);

  for(i=0;i<NUM_FREE_LVHD_BYTES;i++)	/* Rest frei / Headergre 80 Bytes */
    fputc(0,file);

  for(y=0;y<level.fieldy;y++) 
    for(x=0;x<level.fieldx;x++) 
      fputc(Ur[x][y],file);

  fclose(file);

  chmod(filename, LEVEL_PERMS);
}
